@@ -2,8 +2,8 @@ import os
 import pandas
 import selenium
 from time import sleep
-from helper import Helper
-# from .helper import Helper
+# from helper import Helper
+from .helper import Helper
 from selenium import webdriver
 import geckodriver_autoinstaller
 
