@@ -2,14 +2,23 @@ import os
 import pandas
 import selenium
 from time import sleep
+# from helper import Helper
 from .helper import Helper
 from selenium import webdriver
 import geckodriver_autoinstaller
-geckodriver_autoinstaller.install()
 
 
 class Contributing:
 
+    try:
+        geckodriver_autoinstaller.install()
+    except FileNotFoundError:
+        Helper.logo_msg()
+        print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+        print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/\n")
+        exit()
+
+
     def editors(wiki, period, filter, interval):
         try:
 
@@ -68,21 +77,31 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -91,13 +110,18 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def active_editors(wiki, period, filter, interval):
@@ -154,21 +178,31 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -177,13 +211,18 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def edits(wiki, period, filter, interval):
@@ -242,21 +281,31 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -265,13 +314,18 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def user_edits(wiki, period, filter, interval):
@@ -328,21 +382,31 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -351,13 +415,18 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def new_pages(wiki, period, filter, interval):
@@ -416,21 +485,31 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -439,13 +518,18 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def new_registered_users(wiki, period, filter, interval):
@@ -502,21 +586,31 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -525,13 +619,18 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def top_editors(wiki, period, filter, interval):
@@ -593,21 +692,31 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -616,13 +725,18 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def top_edited_pages(wiki, period, filter, interval):
@@ -684,21 +798,31 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -707,13 +831,18 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def active_editors_by_country(wiki, period, filter, interval):
@@ -778,21 +907,31 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -801,10 +940,15 @@ class Contributing:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
