@@ -2,20 +2,29 @@ import os
 import pandas
 import selenium
 from time import sleep
+# from helper import Helper
 from .helper import Helper
 from selenium import webdriver
 import geckodriver_autoinstaller
-geckodriver_autoinstaller.install()
 
 
 class Reading:
 
+    try:
+        geckodriver_autoinstaller.install()
+    except FileNotFoundError:
+        print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+        print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/\n")
+        exit()
+
+
     def total_page_views(wiki, period, filter, interval):
         try:
 
             try:
                 wikis = Helper.get_Wikis_Codes()
             except NameError:
+                Helper.logo_msg()
                 print("\u001b[91mError:\u001b[0m you need to import `Helper` class. Use 'from wikistats2csv import Helper'.\n")
                 exit()
 
@@ -67,21 +76,31 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -90,13 +109,18 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def legacy_page_views(wiki, period, filter, interval):
@@ -154,21 +178,31 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -177,13 +211,18 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def page_views_by_country(wiki, period, filter, interval):
@@ -244,21 +283,31 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -267,13 +316,18 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def unique_devices(wiki, period, filter, interval):
@@ -331,21 +385,31 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -354,13 +418,18 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def top_viewed_articles(wiki, period, filter, interval):
@@ -421,21 +490,31 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -444,10 +523,15 @@ class Reading:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
